use actix_web::error::ErrorBadRequest;
use actix_web::middleware::Logger;
use actix_web::{App, HttpServer, Responder, get, web, Result};
use pronouns_today::InstanceSettings;

#[get("/")]
async fn default(settings: web::Data<InstanceSettings>) -> Result<impl Responder> {
	eprintln!("default");
	let pronouns = settings.select_pronouns(None, None).map_err(ErrorBadRequest)?;
	Ok(format!(include_str!("../static/pronouns.html"), pronouns.render_threeform()))
}

#[get("/{prefs}")]
async fn only_prefs(
	web::Path(prefs): web::Path<String>,
	settings: web::Data<InstanceSettings>,
) -> Result<impl Responder> {
	eprintln!("prefs");
	let pronouns = settings.select_pronouns(None, Some(&prefs)).map_err(ErrorBadRequest)?;
	Ok(format!(include_str!("../static/pronouns.html"), pronouns.render_threeform()))
}

#[get("/{name}/{prefs}")]
async fn prefs_and_name(
	web::Path((name, prefs)): web::Path<(String, String)>,
	settings: web::Data<InstanceSettings>,
) -> Result<impl Responder> {
	eprintln!("prefs+name");
	let pronouns = settings.select_pronouns(Some(&name), Some(&prefs)).map_err(ErrorBadRequest)?;
	Ok(format!(include_str!("../static/pronouns.html"), pronouns.render_threeform()))
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
	env_logger::init();
	HttpServer::new(|| {
		let logger = Logger::default();
		App::new()
			.data(InstanceSettings::default())
			.wrap(logger)
			.service(prefs_and_name)
			.service(only_prefs)
			.service(default)
	})
	.bind("127.0.0.1:8080")?
	.run()
	.await
}
